from .logical_methods import (design_test_batch,
                              find_logical_saboteurs,
                              generate_combinatorial_groups)
from .reports import  plot_batch, generate_batch_report