.logos {
  margin: 0 auto;
}
.logos img {
  width: 250px;
  margin: 30px
}

h1 {
  text-align: center;
}

p {
  font-size: 13px;
}
table.ui.table {
  width: auto;
  text-align: center;
  font-size: 10px;

}

.significant-stats {
  text-align: center;
  display: inline-block;
  float:left;
  margin-right: 1em !important;
  margin-top: 0 !important;
  margin-bottom: 1em;
  padding: 0.5em !important;
}
.page-break {
    page-break-before: always;
}

.table2 th {
  font-size: 7px;
}

#significant-members {
  margin-bottom: 2em;
}
